		.set	BUFFERLEN,	1024
		.set	FILL,		32		## if <FILL bytes free in rcvbuf, send XOFF
		.set	BRKTIME,	5		## ticks to hold a break

		.set	XOFF,		19
		.set	XON,		17

		.set	DATA_REG,	0
		## Transmit/Receive Buffer  (read/write)
		## Baud Rate Divisor LSB if bit 7 of LCR is set  (read/write)
		.set	IER,		1		## interrupt enable register
		## 76543210  2F9, 3F9: Interrupt Enable Register
		##          1 = enable data available int (and 16550 Timeout)
		##         1 = enable THRE interrupt
		##        1 = enable lines status interrupt
		##       1 = enable modem-status-change interrupt
		##   reserved (zero)
		##
		## - 16550 will interrupt if data exists in the FIFO and isn't read
		##   within the time it takes to receive four bytes or if no data is
		##   received within the time it takes to receive four bytes.
		##
		## Baud Rate Divisor MSB if bit 7 of LCR is set  (read/write)
		## - Baud rate divisors can be calculated by taking the oscillating
		##   frequency (1,843,200) and dividing by the quantity of the desired
		##   baud rate times the UART clocking factor (16).  Use the following
		##   formula:
		##   divisor = 1843200 / (BaudRate * 16);
		.set	IIR,		2		## interrupt id register
		## 76543210  2FA, 3FA Interrupt ID Register
		##          1 = no int. pending, 0=int. pending
		##        Interrupt Id bits (see below)
		##       16550  1 = timeout int. pending, 0 for 8250/16450
		##     reserved (zero)
		##   16550  set to 1 if FIFO queues are enabled
		##
		## Bits
		##  21       Meaning            Priority           To reset
		##  00  modem-status-change      lowest      read MSR
		##  01  transmit-register-empty  low         read IIR / write THR
		##  10  data-available           high        read rec buffer reg
		##  11  line-status              highest     read LSR
		##
		## - interrupt pending flag uses reverse logic, 0 = pending, 1 = none
		## - interrupt will occur if any of the line status bits are set
		## - THRE bit is set when THRE register is emptied into the TSR
		##
		## 76543210  2FA, 3FA  FIFO Control Register
		##          1 = enable clear XMIT and RCVR FIFO queues
		##         1 = clear RCVR FIFO
		##        1 = clear XMIT FIFO
		##       1 = change RXRDY & TXRDY pins from mode 0 to mode 1
		##     reserved (zero)
		##   trigger level for RCVR FIFO interrupt
		##
		## Bits      RCVR FIFO
		##  76     Trigger Level
		##  00        1 byte
		##  01        4 bytes
		##  10        8 bytes
		##  11       14 bytes
		##
		## - Bit 0 must be set in order to write to other FCR bits
		## - Bit 1 when set to 1 the RCVR FIFO is cleared and this bit is reset.
		##   The receiver shift register is not cleared.
		## - Bit 2 when set to 1 the XMIT FIFO is cleared and this bit is reset.
		##   The transmit shift register is not cleared.
		.set	LCR,		3		## line control register
		## 76543210  2FB, 3FB  Line Control Register
		##         word length select bits (see below)
		##        0 = 1 stop bit, 1 = 1.5 or 2  (see note)
		##       0 = no parity, 1 = parity (PEN)
		##      0 = odd parity, 1 = even (EPS)
		##     0 = parity disabled, 1 = enabled
		##    0 = turn break off, 1 = force spacing break state
		##   1 = baud rate divisor (DLAB); 0 = RBR, THR or IER
		.set	W5BITS,		0b00000000
		.set	W6BITS,		0b00000001
		.set	W7BITS,		0b00000010
		.set	W8BITS,		0b00000011
		.set	STOPBITS,	2
		.set	PENABLE,	3
		.set	PNONE,		0b00000000
		.set	PODD,		0b00001000
		.set	PEVEN,		0b00011000
		.set	PMARK,		0b00101000
		.set	PSPACE,		0b00111000
		.set	BREAK,		6
		.set	DLAB,		7
		##
		## Bits
		##  10     Word length bits
		##  00 = 5 bits per character
		##  01 = 6 bits per character
		##  10 = 7 bits per character
		##  11 = 8 bits per character
		##
		## - stop bits = 1.5 for 5 bit words or 2 for 6, 7 or 8 bit words
		## - bit 7 changes the mode of registers 3F8 and 3F9.  If set these
		##   registers become the LSB and MSB of the baud rate divisor.
		##   Otherwise 3F8 is the Transmit/Receive Buffer Register and 3F9 is
		##   the Interrupt Enable Register.
		.set	MCR,		4		## modem control register
		## 76543210  2FC, 3FC  Modem Control Register
		##          1 = activate DTR
		##         1 = activate RTS
		##        OUT1
		##       OUT2
		##      0 = normal, 1 = loop back test
		##   reserved (zero)
		##
		## - If bit 4 is set, data from the Transmit Shift Register is received
		##   in the Receiver Shift Register.  The SOUT line is set to logic
		##   high, the SIN line and control lines are disconnected.   CTS, DSR,
		##   RI and CD inputs are disconnected.  DTR, RTS, OUT1 and OUT2 are
		##   then connected internally.
		.set	LSR,		5		## line status register
		## 76543210  2FD, 3FD Line Status Register
		##          1 = data ready
		##         1 = overrun error (OE)
		##        1 = parity error (PE)
		##       1 = framing error (FE)
		##      1 = break interrupt  (BI)
		##     1 = transmitter holding register empty (THRE)
		##    1 = transmitter shift register empty (TSRE)
		##   1 = 16550 PE/FE/Break in FIFO queue, 0 for 8250 & 16450
		.set	DR,			0
		.set	OE,			1
		.set	PE,			2
		.set	FE,			3
		.set	BI,			4
		.set	THRE,		5
		##
		## - Bit 0 is set when a byte is placed in the Receiver Buffer Register
		##   and cleared when the byte is read by the CPU (or when the CPU
		##   clears the FIFO for the 16550).  Results in Receive Data Available
		##   Interrupts if enabled.
		## - Bits 1-4 indicate errors and result in Line Status Interrupts
		##   if enabled.
		## - Bit 1 is set when a second byte is received before the byte
		##   in the Receiver Buffer Register is read by the CPU (the 16550 in
		##   FIFO mode sets this bit when the queue is full and the byte in the
		##   Receiver Shift Register hasn't been moved into the queue).  This
		##   bit is reset when the CPU reads the LSR
		## - Bit 2 is set whenever a byte is received that doesn't match the
		##   requested parity.  Reset upon reading the LSR.  (The 16550 maintains
		##   parity information with each byte and sets bit 2 only when the byte
		##   is at the top of the FIFO queue.)
		## - Bit 3 is set when a character is received without proper stop
		##   bits.  Upon detecting a framing error the UART attempts to
		##   resynchronize.  Reset by reading the LSR.  (The 16550 maintains
		##   this information with each byte and sets bit 3 only when the byte
		##   is at the top of the FIFO queue.)
		## - Bit 4 is set when a break condition is sensed (when space is
		##   detected for longer than 1 fullword).  A zero byte is placed in
		##   the Receiver Buffer Register (or 16550 FIFO).  Reset by reading
		##   the LSR.  (The 16550 maintains this information with each byte and
		##   sets bit 4 only when the byte is at the top of the FIFO queue.)
		## - Bit 5 is set when the Transmit Holding Register shifts a byte
		##   into the Transmit Shift Register (or XMIT FIFO queue is empty for
		##   16550) and is cleared when a byte is written to the THR (or the
		##   XMIT FIFO).   Results in Transmit Holding Register Empty interrupts
		##   if enabled.
		## - Bit 6 is set when both the Transmitter Holding Register and the
		##   Transmitter Shift Register are empty. On the 16550, when the XMIT
		##   FIFO and Transmitter Shift Register are empty.
		## - Bit 7 is 16550 specific and indicates there is a byte in the FIFO
		##   queue that was received with a Parity, Framing or Break error.
		.set	MSR,		6		## modem status register
		## 76543210  2FE, 3FE Modem Status Register
		##          1 = DCTS  Delta CTS  (CTS changed)
		##         1 = DDSR  Delta DSR  (DSR changed)
		##        1 = RI ring indicator changed
		##       1 = DDCD  Delta Data Carrier Detect (DCD changed)
		##      1 = CTS
		##     1 = DSR
		##    1 = ring indicator (RI)
		##   1 = receive line signal detect
		##
		## - Bits 0-3 are reset when the CPU reads the MSR
		## - Bit 4 is the Modem Control Register RTS during loopback test
		## - Bit 5 is the Modem Control Register DTR during loopback test
		## - Bit 6 is the Modem Control Register OUT1 during loopback test
		## - Bit 7 is the Modem Control Register OUT2 during loopback test
.text
## ISR for UART interrupt
start_of_text:
comint:
		## save %edx and %eax as these must be used to check the IIR register
		## of the comm port being serviced.
		pushl	%edx
		pushl	%eax
		## gas seg override bug does not rear it's ugly head here because
		## we're not moving between %eax and direcect memory
		## (eg movl foo,%eax ) which uses the short form of mov to/from the
		## ax register (any size)
		movl	%cs:portIIR,%edx	## get the comm port interrupt
									## %cs and %ds are aliased
		inb		%dx,%al				## identification register and read it
		## test whether it was the commport generating the interrupt. If it
		## was, bit 0 will be clear (and hence the carry flag), and move the
		## interrupt identification bits (1,2) into %al for the indexed
		## function call later.
		shrl	$1,%al				## NC=interrupt, CY=no interrupt
		jnc		1f					## (must chain if carry set)
		## irq was not for us so restore %eax and %edx and then jump to the
		## next isr for this interrupt
		popl	%eax
		popl	%edx
		ljmp	oldirq				## maybe next time ;)
		## load %ds with our data segment (after saving %ds first)
1:		pushl	%ds
		## gas seg override bug does not rear it's ugly head here because
		## we're not moving between %eax and direcect memory
		## (eg movl foo,%eax ) which uses the short form of mov to/from the
		## ax register (any size)
		movw	%cs:ourds,%ds
		## top of while (!(IIR&1))
		## %al already has interrupt type (IIR>>1) from above test (and the
		## bottom of the while loop)
2:		andb	$3,%eax					## incase 16550
		movzbl	%al,%eax
		call	commFuncTable(,%eax,4)	## call the appropriate comm routine
										## IIR==0 (%al==0) -> msr
										## IIR==2 (%al==1) -> xmit
										## IIR==4 (%al==2) -> rcv
										## IIR==6 (%al==3) -> linestat
		## reload the IIR register to see if there is anything more to do to
		## clear the comm port rather than generating unnecessary interrupts
		movl	portIIR,%edx		## point to the interrupt identification
		inb		%dx,%al				## register and read it
		shrl	$1,%al				## NC=interrupt, CY=no interrupt
		jnc		2b					## (continue if carry clear (another
									## pending interrupt))
		## clear the intterupt from the PIC
		movb	$0x20,%al
		outb	%al,$0x20
		## restore the registers and return to the interrupted code
		popl	%ds
		popl	%eax
		popl	%edx
		iret

## Handle Modem Status interrupts
msr:
		movl	portMSR,%edx
		inb		%dx,%al
		movb	%al,_sio_modemstat
		ret

## Handle Line Status interrupts
linestat:
		movl	portLSR,%edx
		inb		%dx,%al
		movb	%al,_sio_linestat
		bt		$BI,%al				## test for break
		jnc		1f
		movb	$1,_sio_break
1:		ret

## increment buffer pointer circularly
## %ebx holds buffer pointer and %ecx holds the offset to increment by
circadd:
		addl	%ecx,%ebx
1:		cmpl	$BUFFERLEN,%ebx
		jb		2f
		subl	$BUFFERLEN,%ebx
		jmp		1b
2:		ret

## Attempt to transmit character
xmit:
		## test the 
		movl	portLSR,%edx
		inb		%dx,%al
		bt		$THRE,%al			## tx register empty
		jc		1f
		ret							## tx register not empty, false alarm
		## check to see if an xoff should be sent
1:		testb	$0xff,sendxoff
		jz		2f
		## if so, send the 'xoff' and set the 'sent xoff' flag after clearing
		## the 'send xoff' flag
		movb	$0,sendxoff
		movb	$XOFF,%al
		movl	portTHR,%edx
		outb	%al,%dx
		movb	$1,sentxoff
		ret							## nothing more to do here
		## check to see if an xon should be sent
2:		testb	$0xff,sendxon
		jz		3f
		## if so, send the 'xon' and clear the 'sent xoff' flag after clearing
		## the 'send xon' flag
		movb	$0,sendxon
		movb	$XON,%al
		movl	portTHR,%edx
		outb	%al,%dx
		movb	$0,sentxoff
		ret							## nothing more to do here
		## save %ebx and %ecx as we use these registers
3:		pushl	%ebx
		pushl	%ecx
		## if we're doing software flow control and an xoff was received, or
		## there are no more characters to be sent, then exit
		testb	$0xff,_sio_doxoff	## s/w flow control
		jz		4f
		testb	$0xff,gotxoff		## an xoff was received
		jnz		5f
4:		movl	xmittail,%ebx		## load the tail pointer into %ebx
		cmpl	xmithead,%ebx		## is the buffer empty
		jne		6f					## not equal means chars in buffer
		## we cannot send another character right now, so restore %ecx and
		## %ebx and return
5:		popl	%ecx
		popl	%ebx
		ret
		## get a char from the transmit buffer
6:		movb	xmitbuf(%ebx),%al	## %ebx loaded previously
		## point to the next character in the transmit buffer
		movl	$1,%ecx				## xmittail++
		call	circadd				## with wrap at buffer size
		movl	%ebx,xmittail
		## send the character
		movl	portTHR,%edx
		outb	%al,%dx
		## restor %ecx and %ebx and exit
		popl	%ecx
		popl	%ebx
		ret

testbuf:
## tests for buffer overflow or underflow
## call with:
##	%eax = head (overflow) tail (underflow)
##	%edx = tail (overflow) head (underflow)
##	%ecx = offset for circadd
## 
## clobbers %ebx (holds result of circadd, initially assigned from %eax)
## carry set=buffer overflow/underflow
## carry clear=buffer ok
## base logic:
##	(oflow) head<tail && (temp>=tail || temp<head) || temp>=tail && temp<head
##	(uflow) tail<head && (temp>=head || temp<tail) || temp>=head && temp<tail
## comments assume testing for overflow
		movl	%eax,%ebx
		call	circadd
		cmpl	%edx,%eax		## head < tail
		jae		1f				## jump if tail >= head (sc on failure &&)
		cmpl	%edx,%ebx		## temp >= tail
		jae		2f				## jump if temp >= tail (sc on success ||)
		cmpl	%eax,%ebx		## temp < head
		jb		2f				## jump if temp < head  (sc on success ||)
		## previous conditions failed
1:		cmpl	%edx,%ebx		## temp >= tail
		jb		3f				## jump if tail >= temp (sc on failure &&)
		cmpl	%eax,%ebx		## temp < head
		jae		3f				## jump if head >= tail (sc on failure &&)
		## buffer overflow (or underflow if %eax and %edx reversed)
2:		stc
		ret
		## buffer does not overflow (or underflow if %eax and %edx reversed)
3:		clc
		ret

##Attempt to receive a character
rcv:
		movl	portLSR,%edx
		inb		%dx,%al
		bt		$DR,%al			## rx register full
		jc		1f
		ret						## rx register not full, false alarm
1:		pushl	%ebx			## save %ebx and %ecx
		pushl	%ecx
		movl	portRBR,%edx
		inb		%dx,%al			## read in the received character. always has
								## to be done to clear the uart interrupt
								## register and prevent overrun errors
		## tuck the char away in the receive buffer. There is always a hole
		## that is never used, even if the buffer is full because head points
		## to the next available slot, and if the hole was not available, head
		## would wind up equalling tail, which implies an empty buffer
		movl	rcvhead,%ebx
		movb	%al,rcvbuf(%ebx)
		## always flag XOFF and XON for flow control (xmit chooses weather or
		## not to ignore them) even if the buffer is full
		cmpb	$XOFF,%al
		jne		7f
		movl	$1,gotxoff
		jmp		8f
7:		cmpb	$XON,%al
		jne		8f
		movl	$0,gotxoff
8:		## test if receive buffer is full
		movl	rcvhead,%eax	## testing for overflow so load head into %eax
		movl	rcvtail,%edx	## and tail into %edx
		movl	$1,%ecx			## one byte to check
		call	testbuf			## cy=full, nc=not full
		jnc		2f
		## buffer overflow
		movl	$1,_sio_error
		incl	_sio_errct
		popl	%ecx
		popl	%ebx
		ret
		## the buffer is not full, so save the new head pointer (%ebx was set
		## up by the call to testbuf).
2:		movl	%ebx,rcvhead
		## test if less that 'FILL' bytes free in receive buffer (%eax and
		## %edx are already initialized to rcvhead and rcvtail, respectively)
		movl	$FILL,%ecx		## FILL bytes to check
		call	testbuf			## cy=full, nc=not full
		jnc		4f
		testb	$0xff,_sio_doxoff
		jz		4f
		## send an xoff character
		movb	$1,sendxoff
		## if the transmit register is empty, send the xoff char, otherwise
		## xmit will do nothing this time but xoff will be sent as soon as the
		## character in the register has been sent.
		call	xmit
4:		popl	%ecx
		popl	%ebx
		ret	

## User interface routines
## write a character. returns 0 if successful, 1 if transmit buffer is full
_sio_put:.globl	_sio_put
		pushl	%ebx
		movl	xmithead,%eax	## testing for overflow so load head into %eax
		movl	xmittail,%edx	## and tail into %edx
		movl	$1,%ecx			## one byte to check
		call	testbuf
		jnc		1f				## no carry if no overflow
		popl	%ebx
		movl	$1,%eax
		ret
1:		movb	8(%esp),%dl		## get the character to be sent
		movb	%dl,xmitbuf(%eax)	## and put it in the buffer
		movl	%ebx,xmithead	## %ebx has been initialized by testbuf above
		popl	%ebx
		## Call xmit in case transmitter has been idle for a while. If
		## transmitter is busy, xmit won't do anything
		call	xmit
		movl	$0,%eax
		ret

## Check for character available (1 means char ready)
_sio_charready:.globl	_sio_charready
		## if rcvtail == rcv head then there are no characters in the buffer
		movl	rcvtail,%eax
		cmpl	%eax,rcvhead
		setne	%al
		movzbl	%al,%eax
		ret

## Wait for character and return it. returns -1 for break
_sio_get:.globl	_sio_get
		pushl	%ebx			## save %ebx (for gcc)
		##	while (rcvtail==rcvhead) {
		movl	rcvtail,%ebx	## if rcvtail == rcv head then there are no
1:		cmpl	rcvhead,%ebx	## characters in the buffer
		jne		2f
		##		if (_sio_brkmode
		testb	$0xff,_sio_brkmode
		jz		1b
		##						&& _sio_break) {
		testb	$0xff,_sio_break
		jz		1b
		movl	$0,_sio_break
		##			return -1
		movl	$-1,%eax
		pushl	%eax			## for pop at function exit
		jmp		3f
		##		}
		##	}
		## a character is available (%ebx alreadly loaded with rcvtail)
2:		movb	rcvbuf(%ebx),%al
		movzbl	%al,%eax
		movl	$1,%ecx
		call	circadd
		movl	%ebx,rcvtail
		pushl	%eax			## save received character on the stack
		## if an xoff was not sent then there is no need to check weather or
		## not to send an xon
		testb	$0xff,sentxoff
		jz		3f
		## test if less than FILL bytes remaining in buffer
		movl	rcvhead,%edx	## testing for underflow so load head into
		movl	rcvtail,%eax	## %edx and tail into %eax
		movl	$FILL,%ecx		## FILL bytes to check
		call	testbuf
		jnc		3f				## more than FILL chars remaining if no carry
		## send an xon character
		movb	$1,sendxon
		## if the transmit register is empty, send the xon char, otherwise
		## xmit will do nothing this time but xon will be sent as soon as the
		## character in the register has been sent.
		call	xmit
3:		popl	%eax			## restore the received char from the stack
		popl	%ebx			## restore %ebx (for gcc)
		ret

## Send a break
_sio_sendbreak:.globl _sio_sendbreak
		movl	4(%esp),%ecx	## number of character cells to hold break
		## wait until the transmitter is empty
		movl	portLSR,%edx
1:		inb		%dx,%al
		bt		$THRE,%al
		jnc		1b
		## set the break control bit of the line control register
		movl	portLCR,%edx
		inb		%dx,%al
		bts		$BREAK,%al
		outb	%al,%dx
		## send a character (anything will do(?)) for timing purposes
2:		movl	portTHR,%edx
		outb	%al,%dx
		## wait until the transmitter is empty (the bogus character has been
		## sent)
		movl	portLSR,%edx
3:		inb		%dx,%al
		bt		$THRE,%al
		jnc		3b
		loop	2b				## ecx holds the number of `characters' to
								## send while holding the break condition
		## clear the break condition
		movl	portLCR,%edx
		inb		%dx,%al
		btr		$BREAK,%al
		outb	%al,%dx
		ret

# the following two functions provide access to the divisor latch registers
# in a convinient fassion.  HOWEVER: the program must do the translation
# to/from baud as these functions DO NOT.  This allows the program to set
# strange baud rates or control the allowable rates rather than putting
# restrictions on the program.
_sio_setspeed:	.globl	_sio_setspeed
		## sets the divisor latch to the value passed on the stack.  DOES NOT
		## convert from baud.
		movl	portLCR,%edx
		inb		%dx,%al
		bts		$DLAB,%al		## access the divisor latch registers
		outb	%al,%dx
		movw	4(%esp),%ax
		movl	portDLL,%edx
		outb	%al,%dx			## set LSB of divisor
		movl	portDLM,%edx
		movb	%ah,%al
		outb	%al,%dx			## set MSB of divisor
		movl	portLCR,%edx
		inb		%dx,%al
		btr		$DLAB,%al		## access data/interrupt enable registers
		outb	%al,%dx
		ret

_sio_getspeed:	.globl	_sio_getspeed
		## gets the divisor latch.  DOES NOT convert from to baud.
		movl	portLCR,%edx
		inb		%dx,%al
		bts		$DLAB,%al		## access the divisor latch registers
		outb	%al,%dx
		movl	portDLM,%edx
		inb		%dx,%al			## get MSB of divisor
		movl	portDLL,%edx
		movb	%al,%ah
		inb		%dx,%al			## get LSB of divisor
		pushl	%eax
		movl	portLCR,%edx
		inb		%dx,%al
		btr		$DLAB,%al		## access data/interrupt enable registers
		outb	%al,%dx
		popl	%eax
		ret

# the following two functions provide direct access to the modem control
# register. NOT RECOMMENDED but provided for when direct access is required
_sio_setmcr:	.globl	_sio_setmcr
		movl	portMCR,%edx
		movb	4(%esp),%al
		outb	%al,%dx
		ret

_sio_getmcr:	.globl	_sio_getmcr
		movl	portMCR,%edx
		xorl	%eax,%eax
		inb		%dx,%al
		ret

# the following two functions provide direct access to the line control
# register. NOT RECOMMENDED but provided for when direct access is required
_sio_setlcr:	.globl	_sio_setlcr
		movl	portMCR,%edx
		movb	4(%esp),%al
		outb	%al,%dx
		ret

_sio_getlcr:	.globl	_sio_getlcr
		movl	portLCR,%edx
		xorl	%eax,%eax
		inb		%dx,%al
		ret

# set type communications paramters (but not the baud rate)
_sio_setparms:	.globl	_sio_setparms
		## assumes enums defined in serio.h were used.  the enums set the
		## correct bits in the control byte and can be just or'ed together
		## and output to the line control regitster.
		movb	4(%esp),%al		# bits per word
		orb		8(%esp),%al		# parity
		orb		12(%esp),%al	# stop bits
		movl	portLCR,%edx
		andb	$0b00111111,%al
		outb	%al,%dx
		ret
end_of_text:

# attempt to lock down the memory used by this module. returns 0 if successful
# and -1 if unsuccessful.  Called by sio_init()
sio_lock:
		movw	%ds,%bx
		movw	%bx,ourds

		# get ds base
		movzwl	%bx,%ebx		# selector
		movw	$0x0006,%ax
		int		$0x31
		jnc		1f
err:	movl	$-1,%eax
		jmp		sio_exit
		# cx:dx base
1:		movw	%cx,%bx
		shll	$16,%ebx
		movw	%dx,%bx
		movl	%ebx,%edx		# base

		# lock text
		# bx:cx address
		# si:di size
		leal	start_of_text(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$end_of_text,%eax
		subl	$start_of_text,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x600,%eax
		int		$0x31
		jc		err

		# lock data
		leal	start_of_data(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$end_of_data,%eax
		subl	$start_of_data,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x600,%eax
		int		$0x31
		jc		err
		xorl	%eax,%eax
sio_exit:
		ret


# attempt to unlock the memory used by this module. returns 0 if successful
# and -1 if unsuccessful.  Called by sio_uninit()
sio_unlock:
		# unlock memory
		# get ds base
		movw	%ds,%bx
		movzwl	%bx,%ebx		# selector
		movw	$0x0006,%ax
		int		$0x31
		jnc		1f
err2:	movl	$-1,%eax
		jmp		sio_exit
		# cx:dx base
1:		movw	%cx,%bx
		shll	$16,%ebx
		movw	%dx,%bx
		movl	%ebx,%edx		# base

		# unlock text
		# bx:cx address
		# si:di size
		leal	start_of_text(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$end_of_text,%eax
		subl	$start_of_text,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x601,%eax
		int		$0x31
		jc		err2

		# unlock data
		leal	start_of_data(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$end_of_data,%eax
		subl	$start_of_data,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x601,%eax
		int		$0x31
		jc		err2
		xorl	%eax,%eax
		jmp		sio_exit

# sets up the UART register addresses. separate variables are used for each
# register for both readability and speed(? removes need for adding an offset
# to dx, movl portLCR,%edx; inb %dx,%al will read the line control register)
sio_setbase:
		movl	%ebx,basead
		movl	%ebx,portDLL
		movl	%ebx,portTHR
		movl	%ebx,portRBR
		incl	%ebx
		movl	%ebx,portDLM
		movl	%ebx,portIER
		incl	%ebx
		movl	%ebx,portIIR
		incl	%ebx
		movl	%ebx,portLCR
		incl	%ebx
		movl	%ebx,portMCR
		incl	%ebx
		movl	%ebx,portLSR
		incl	%ebx
		movl	%ebx,portMSR
		ret

# hooks the interrupt associated with the IRQ number in ebx. saves previous
# vector for chaining and restoring at module cleanup.
# also sets the IRQ masks for the PICs
sio_hookinterrupt:
		pushl	%ebx
		# save old interrupt vector
		# cx:edx address (sel:offs)
		movzbl	irqints(%ebx),%ebx
		movw	$0x0204,%ax
		int		$0x31
		movl	%edx,old_offs
		movw	%cx,old_sel

		# hook interrupt
		# cx:edx address (sel:offs)
		movw	%cs,%cx
		movl	$comint,%edx
		movw	$0x0205,%ax
		int		$0x31
		popl	%ebx
		inb		$0xa1,%al
		movb	%al,%ah
		inb		$0x21,%al
		movw	%ax,%cx
		andw	irqemasks(,%ebx,2),%ax	# calculate value to enable this irq
		andw	irqdmasks(,%ebx,2),%cx	# calculate value to disable this irq
										# for cleanup
		movw	%cx,irqmask				# this will be or'ed with the PIC
										# masks in unhookinterrupt to put this
										# mask in the same state it was in
										# when this module is inited.
		outb	%al,$0x21
		movb	%ah,%al
		outb	%al,$0xa1
		ret

# restore the interrupt vector for the irq in ebx and restores the PIC int
# enable mask to what it was at startup (this allows the program to hook the
# mouse comport for its own uses without killing the mouse on exit)
sio_unhookinterrupt:
		pushl	%ebx
		inb		$0xa1,%al
		movb	%al,%ah
		inb		$0x21,%al
		orw		irqmask,%ax		# restore the PIC mask for this irq to what it
								# was when this module was initialize
		outb	%al,$0x21
		movb	%ah,%al
		outb	%al,$0xa1
		# unhook interrupt
		# cx:edx address (sel:offs)
		movzbl	irqints(%ebx),%ebx
		movw	old_sel,%cx
		movl	old_offs,%edx
		movw	$0x0205,%ax
		int		$0x31
		popl	%ebx
		ret

# initializes this module. takes base address and irq number of the com port
# (ie int sio_init(base address, irq number) ). Locks memory, hooks the
# interrupt and enables DTR/RTS
_sio_init:	.globl	_sio_init
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%ebx
		pushl	%esi
		pushl	%edi
		movl	8(%ebp),%ebx
		call	sio_setbase
		call	sio_lock
		orl		%eax,%eax
		jnz		1f
		movzbl	12(%ebp),%ebx
		movb	%bl,irqnum
		call	sio_hookinterrupt
		movl	portMCR,%edx
		inb		%dx,%al
		movb	%al,%ah
		movb	$0x0b,%al
		outb	%al,%dx
		movl	portIER,%edx
		inb		%dx,%al
		movw	%ax,cntlRegs
		movb	$0x0f,%al
		outb	%al,%dx
		xorl	%eax,%eax
1:		popl	%edi
		popl	%esi
		popl	%ebx
		popl	%ebp
		ret

# cleans up after use (call befor program exit). unlocks memory, restores
# interrupt vector and clears DTR/RTS
_sio_uninit:	.globl	_sio_uninit
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%ebx
		pushl	%esi
		pushl	%edi
		movl	portIER,%edx
		movw	cntlRegs,%ax
		outb	%al,%dx
		movl	portMCR,%edx
		movb	%ah,%al
		outb	%al,%dx
		movzbl	irqnum,%ebx
		call	sio_unhookinterrupt
		call	sio_unlock
		popl	%edi
		popl	%esi
		popl	%ebx
		popl	%ebp
		ret
		
.data
start_of_data:
basead:								## base address of UART I/O
portDLL:							## divisor latch LSB
portTHR:							## transmit holding register
portRBR:.long	0					## receiver buffer register
portDLM:							## divisor latch MSB
portIER:.long	0					## interrupt enable register
portIIR:.long	0					## interrupt identification register
portLCR:.long	0					## line control register
portMCR:.long	0					## modem control register
portLSR:.long	0					## line status register
portMSR:.long	0					## modem status register

ourds:	.word	0
irqmask:.word	0					## IRQ & mask bit for PIC
irqnum:	.byte	0
		.byte	0
cntlRegs:.word	0
oldirq:
old_offs:.long	0					## offset
old_sel:.long	0					## selector

## Define two circular buffers
xmitbuf:.fill	BUFFERLEN
rcvbuf:	.fill	BUFFERLEN
xmithead:.long	0
xmittail:.long	0
rcvhead:.long	0
rcvtail:.long	0

sentxoff:.byte	0
sendxoff:.byte	0
sendxon:.byte	0
gotxoff:.byte	0

commFuncTable:
		.long	msr
		.long	xmit
		.long	rcv
		.long	linestat

## global variables of interest to user's program
## error words
_sio_error:		.globl	_sio_error
		.long	0
_sio_errct:		.globl	_sio_errct
		.long	0
_sio_break:		.globl	_sio_break
		.long	0					## set to 1 when break occurs
_sio_linestat:	.globl	_sio_linestat
		.long	0					## line status
_sio_modemstat:	.globl	_sio_modemstat
		.long	0					## modem status
_sio_doxoff:		.globl	_sio_doxoff
		.long	1					## set xon/xoff mode on
_sio_brkmode:	.globl	_sio_brkmode
		.long	0					## if 1, _sio_get() returns -1 if break occured
end_of_data:
irqemasks:
		## masks for enableing PIC interrupts. IRQ 2 is automatically enabled
		## whenever IRQ 8-15 is enabled. Recomended useage:
		##		inb		$0xa1,%al				# read slave int enable mask
		##		movb	%al,%ah					# irq8-15 in bits 8-15 of ax
		##		inb		$0x21,%al				# read master int enable mask
		##										# irq0-7 in bits 0-7 of ax
		##		andw	%ax,irqemasks(,%ebx,2)	# ebx holds irq number
		##		outb	%al,$0x21				# write master int enable mask
		##		movb	%ah,$al
		##		outb	%al,$0xa1				# write slave int enable mask
		##
		## related info: with the 8259 PIC a 1 in the interrupt enable mask
		## DISABLES the corresponding irq (ie a 1 in bit 3 disables irq 3, or
		## 11 if the PIC is the slave) and a 0 ENABLES the irq.
		.word	0b1111111111111110
		.word	0b1111111111111101
		.word	0b1111111111111011
		.word	0b1111111111110111
		.word	0b1111111111101111
		.word	0b1111111111011111
		.word	0b1111111110111111
		.word	0b1111111101111111
		.word	0b1111111011111011
		.word	0b1111110111111011
		.word	0b1111101111111011
		.word	0b1111011111111011
		.word	0b1110111111111011
		.word	0b1101111111111011
		.word	0b1011111111111011
		.word	0b0111111111111011
irqdmasks:
		## masks for disabling PIC interrupts. IRQ2 is not disabled by
		## disabling IRQ's 8-15 as this would cause the other IRQ's in
		## this range to be disabled, which would be a BAD THING (would kill
		## the hard drive, although I think the bios always ensures the HD
		## IRQ is enabled every time it's needed.
		.word	0b0000000000000001
		.word	0b0000000000000010
		.word	0b0000000000000100
		.word	0b0000000000001000
		.word	0b0000000000010000
		.word	0b0000000000100000
		.word	0b0000000001000000
		.word	0b0000000010000000
		.word	0b0000000100000000
		.word	0b0000001000000000
		.word	0b0000010000000000
		.word	0b0000100000000000
		.word	0b0001000000000000
		.word	0b0010000000000000
		.word	0b0100000000000000
		.word	0b1000000000000000
irqints:
		## irq to interrupt translation.  Assumes standard setup of PICs
		## (guaranteed(?) under dpmi)
		.byte	0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f
		.byte	0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77
